package ch.unige.solidify.controller;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.hateoas.RepresentationModel;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.repository.SolidifyRepositoryNoSql;
import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.rest.ResourceNoSql;
import ch.unige.solidify.rest.Tool;

public abstract class ResourceNoSqlReadOnlyController<T extends ResourceNoSql<T>>
        extends SolidifyController implements ControllerWithHateoasHome {

  protected SolidifyRepositoryNoSql<T> itemRepository;

  public ResourceNoSqlReadOnlyController(SolidifyRepositoryNoSql<T> itemRepository) {
    this.itemRepository = itemRepository;
  }

  @GetMapping(SolidifyConstants.URL_ID)
  public HttpEntity<T> get(@PathVariable String id) {
    final T t = this.itemRepository.findOne(this.getIndex(), id);
    if (t != null) {
      this.addLinks(t);
      return new ResponseEntity<>(t, HttpStatus.OK);
    }
    return new ResponseEntity<>(HttpStatus.NOT_FOUND);
  }

  @GetMapping
  public HttpEntity<Collection<T>> list(@ModelAttribute T search, Pageable pageable) {
    final Page<T> listItem = this.itemRepository.findAll(this.getIndex(), search, pageable);
    return this.page2Collection(listItem, pageable);
  }

  protected void addLinks(T t) {
    t.removeLinks();
    t.addLinks(linkTo(this.getClass()));
  }

  protected <W extends RepresentationModel<W>> void addOthersLinks(W w) {
    // Do nothing
    // Could be overridden by subclass
  }

  protected abstract String getIndex();

  protected HttpEntity<Collection<T>> page2Collection(Page<T> listItem, Pageable pageable) {
    for (final T t : listItem) {
      this.addLinks(t);
    }
    final Collection<T> collection = new Collection<>(listItem, pageable);
    collection.add(linkTo(this.getClass(), pageable).withSelfRel());
    collection.add(Tool.parentLink((linkTo(this.getClass(), pageable)).toUriComponentsBuilder())
            .withRel(ActionName.MODULE));
    this.addSortLinks(linkTo(this.getClass(), pageable), collection);
    this.addPageLinks(linkTo(this.getClass(), pageable), collection, pageable);
    this.addOthersLinks(collection);
    return new ResponseEntity<>(collection, HttpStatus.OK);
  }

}
